/************** BEST VIEWED IN A CODE EDITOR 80 COLUMNS WIDE *******************
*
* Milwaukee OMT Blade Holder
* Benjamen Johnson <workshop.electronsmith.com>
*
* Created 20210312
* openSCAD Version: 2019.05 
* Licesnse: Creative Commons - Attribution - Share Alike
*           https://creativecommons.org/licenses/by-sa/4.0/
*******************************************************************************/
/*[Hidden]*/
ver = "ELECTRONSMITH V1.0";

/*[Post Parameters]*/
/******************************************************************************
* The posts are what hold the OMT blades
*******************************************************************************/
// Distance between the posts (zero for one post)
post_dist = 35;

// Diameter of the post
post_dia = 9;

// Post height
post_height = 24.5;

//how far up from the base the post starts
post_pos = 9.5;

// Thickness of the keys that stop the blade from rotating
key_thickness = 1.5;

// Width of the keys that stop the blade from rotating
key_width = 3;

/*[Holder Parameters]*/
/*******************************************************************************
*    /\ /\ /\ /\ /\ /\ /\ /\ /\ /\  < ---- bits
*   -------------------------------  --
*  |                               |  body_height 
*  |                               |
*   -------------------------------  --
*  |< ------- body_width --------->|
*   -------------------------------  --
*  | X  X  X  X  X  X  X  X  X  X  |  body_depth
*   -------------------------------  --
*******************************************************************************/
// Width of the holder body
body_width = 82;

// Depth of holder 
body_depth = 14; // 11

// Height of holder
body_height = 20;

// How much to chamfer the bottom sides (DO NOT TOUCH!!!)
side_chamfer = 5;

// How much to chamfer the bottom front and back
front_chamfer = 4;

// How far to cut into the body of the holder
cutout_depth = 10.5;

/*[Connector Parameters]*/
/*******************************************************************************
*     -------------------------------
*   -| X  X  X  X  X  X  X  X  X  X  |-
*     -------------------------------
*  | <------ total_pin_length -------->|
*
*    -------
*   |---O---|  o <- pin_dia (diamter of pivot pin)
*   |   |   | 
*   |   |   |    <- catch_dia (diameter of the semicircular catches)
*    -------
******************************************************************************/
// Length of cylinder passing through the holder that it rotates on
total_pin_length = body_width+6; // 88

// Diameter of the above cylinder
pin_dia = 4.5; // 4.5

// Pin distance from the bottom of holder;
pin_height = 10;

// Diameter of the semicircular catches
catch_dia = 2.5;

/*[Stamp Options]*/
/*******************************************************************************
* Stamp Options
*******************************************************************************/
// Which font do you want to use
font = "REGISTRATION PLATE UK:style=Bold";

// How big you want the text
font_size = 5;//[0:0.5:10]

// How thick the font is (back to front)
font_thickness = 0.6;

/*[Hidden]*/
/*******************************************************************************
* Array Building and Calculations
*******************************************************************************/
// Add a slight offset to cuts to make model render properly
$dl = 0.01; //0.005

// define zero positions
x_zero = -body_width/2;
y_zero = -body_height/2;
z_zero = -body_depth/2;

// matrix to invert the X coord
INVERT_X = [[-1,0,0],[0,1,0],[0,0,1]];    
INVERT_Z = [[1,0,0],[0,1,0],[0,0,-1]];    

/*******************************************************************************
* Make it
*******************************************************************************/
translate([0,0,body_depth/2])
union(){
    difference(){
        color("red")
                make_holder();
        translate([0,font_thickness-$dl,0])
            rotate([90,0,0])
                stamp_version(ver,font_size,font_thickness);
    } //end difference
} //end difference

/*******************************************************************************
* Make the holder
*******************************************************************************/
module make_holder(){

    union(){
        translate([0,body_height/2,0])
        difference(){
            MilwaukeeHolder();
            
            // how much to cut out of the basic holder
            min_edge = 5;
            width2= body_width - 2 * min_edge;
            
            // cut out middle
            translate([0,(body_height-cutout_depth)/2,0])
            cube([width2,cutout_depth+$dl,body_depth+$dl],center=true);

        }//end difference
 
        // make connectors
        translate([-post_dist/2,post_pos+post_height/2,0])
        OMT_connector();    

        translate([post_dist/2,post_pos+post_height/2,0])
        OMT_connector();    
       
    }// end union
} // end module

/*******************************************************************************
* Make the part the that OMT blades fit ontp
*******************************************************************************/
module OMT_connector() {
    union(){
        rotate([90,0,0])
        cylinder(d=post_dia,h=post_height,center=true,$fn=50);
            
        cube([post_dia+2*key_width,post_height,key_thickness],center=true);
            
        translate([0,0,-(post_dia/2+key_width/2-0.5)])
        cube([key_thickness,post_height,key_width],center=true);
    } //end union
} // end module

/*******************************************************************************
* Module for creating the body of the Milwuakee holder
*******************************************************************************/
module MilwaukeeHolder(){
   
// calculate where to put the pin
pin_pos = y_zero+pin_height;

// Calculate important vectors
MAIN_BODY = [body_width,body_height,body_depth];
  
PIN_CUTOFF = [pin_dia,pin_dia,pin_dia];
PIN_CUTOFF_POS =  [total_pin_length/2+1.4,0,-pin_dia/2];
    
HORIZ_CATCH_POS = [body_width/2,pin_pos,0];
VERT_CATCH_POS = [body_width/2,0,0];
    
SIDE_CHAMFER_POS=[(body_width+catch_dia)/2,y_zero,0]; 
SIDE_CHAMFER_CUT = [side_chamfer,side_chamfer,body_depth+$dl];

FRONT_CHAMFER_POS = [0,y_zero,z_zero];    
FRONT_CHAMFER_CUT =[body_width+$dl, front_chamfer, front_chamfer];
  
    difference(){
        union(){
            // Form the main body
            cube(MAIN_BODY,center = true);
                    
            // form the pivot pins
            translate([0,pin_pos,0])
            difference(){
                rotate([0,90,0])
                    cylinder(d=pin_dia,h=total_pin_length,center=true,$fn=50);
                
                // cut off end at 45 degrees
                translate(PIN_CUTOFF_POS)
                    rotate([0,45,0])
                        cube(PIN_CUTOFF,center=true);
                    
                // cut off other end at 45 degrees
                translate(PIN_CUTOFF_POS*INVERT_X)
                    rotate([0,45,0])
                        cube(PIN_CUTOFF,center=true);        
            } // end difference
            
            //form the horizontal catches
            translate(HORIZ_CATCH_POS)
                cylinder(d=catch_dia,h=body_depth,center=true,$fn=50);
            
            translate(HORIZ_CATCH_POS*INVERT_X)
                cylinder(d=catch_dia,h=body_depth,center=true,$fn=50);
            
            //form the vertical catches
            translate(VERT_CATCH_POS)
                rotate([90,0,0])
                    cylinder(d=catch_dia,h=body_height,center=true,$fn=50);
            
            translate(VERT_CATCH_POS*INVERT_X)
                rotate([90,0,0])
                    cylinder(d=catch_dia,h=body_height,center=true,$fn=50);
        } // end union
        
        //chamfer bottom sides
        translate(SIDE_CHAMFER_POS)
            rotate([0,0,45])
                cube(SIDE_CHAMFER_CUT,center=true);
        
        translate(SIDE_CHAMFER_POS*INVERT_X)
            rotate([0,0,45])
                cube(SIDE_CHAMFER_CUT,center=true);
        
        //chamfer bottom front and back 
        translate(FRONT_CHAMFER_POS)
            rotate([45,0,0])
                cube(FRONT_CHAMFER_CUT,center=true);
        
        translate(FRONT_CHAMFER_POS*INVERT_Z)
            rotate([45,0,0])
                cube(FRONT_CHAMFER_CUT,center=true);
    
    }//end difference
} // end module

/*******************************************************************************
* Module for putting the version number on the print
*******************************************************************************/
module stamp_version(version_text,size,thickness){
    color("white")
    linear_extrude(height = thickness)
        text(text = version_text, font = font, size = size, valign="center",halign = "center");
} // end module